#include <vector>
#include "conflictcheck_thread.h"
#include "epgsearchcfg.h"

cConflictCheckThread *cConflictCheckThread::m_Instance = NULL;
time_t cConflictCheckThread::m_cacheNextConflict = 0;
int cConflictCheckThread::m_cacheRelevantConflicts = 0;
int cConflictCheckThread::m_cacheTotalConflicts = 0;

cConflictCheckThread::cConflictCheckThread(cPluginEpgsearch* thePlugin)
: cThread("EPGSearch: conflictcheck")
{
    m_plugin = thePlugin;
    m_Active = false;
    m_lastUpdate = time(NULL);
}

cConflictCheckThread::~cConflictCheckThread() {
    if (m_Active) 
	Stop();
}

void cConflictCheckThread::Init(cPluginEpgsearch* thePlugin) 
{
    if (EPGSearchConfig.checkTimerConflictsAfterUpdate || EPGSearchConfig.conflictCheckIntervall == 0)
	return;    
    if (m_Instance == NULL) {
	m_Instance = new cConflictCheckThread(thePlugin);
	m_Instance->Start();
    }
}

void cConflictCheckThread::Exit(void) {
    if (m_Instance != NULL) {
	m_Instance->Stop();
	DELETENULL(m_Instance);
    }
}

void cConflictCheckThread::Stop(void) {
    m_Active = false;
    Cancel(6);
}

void cConflictCheckThread::Action(void) 
{
    SetPriority(19);

    m_Active = true;
    // let VDR do its startup
    sleepSec(20);
    time_t nextUpdate = time(NULL);
    while (m_Active) 
    {
	time_t now = time(NULL);
	if (now >= nextUpdate)
	{
	    if (Timers.BeingEdited()) 		
	    {
		sleepSec(1);
		continue;
	    }
	    isyslog("EPGSearch: timer conflict check started");
	    LogFile.Log(1,"timer conflict check started");

	    cConflictCheck conflictCheck;   
	    conflictCheck.Check(); 

	    time_t nextConflict = 0;
	    if (conflictCheck.relevantConflicts > 0)
	    {
		char* msgfmt = NULL;
		asprintf(&msgfmt, tr("%d timer conflict(s)! First at %s. Show them?"), conflictCheck.relevantConflicts,
			 *DateTime(conflictCheck.nextRelevantConflictDate));
		if (SendMsg(msgfmt, true,7) == kOk)			
		{
		    m_plugin->showConflicts = true;
		    cRemote::CallPlugin("epgsearch");	
		}
		free(msgfmt);
	    }
	    
	    // store for external access
	    cConflictCheckThread::m_cacheNextConflict = conflictCheck.nextRelevantConflictDate;
	    cConflictCheckThread::m_cacheRelevantConflicts = conflictCheck.relevantConflicts;
	    cConflictCheckThread::m_cacheTotalConflicts = conflictCheck.numConflicts;

	    isyslog("EPGSearch: timer conflict check finished");
	    LogFile.Log(1,"timer conflict check finished");

	    m_lastUpdate = time(NULL);
	    int Intervall = EPGSearchConfig.conflictCheckIntervall;
	    if (nextConflict > 0 && EPGSearchConfig.conflictCheckWithinLimit > 0 && 
		nextConflict - time(NULL) < EPGSearchConfig.conflictCheckWithinLimit * 60)
		Intervall = EPGSearchConfig.conflictCheckIntervall2;

	    nextUpdate = long(m_lastUpdate/60)*60 + (Intervall * 60);
	}
	sleepSec(2); // to avoid high system load if time%30==0 
	while (m_Active && time(NULL)%30 != 0) // sync heart beat to a multiple of 5secs
	    sleepSec(1);
    };
    isyslog("EPGSearch: Leaving conflict check  thread");
}

