#include <vector>
#include "epgsearchsetup.h"
#include "epgsearchcfg.h"
#include "searchtimer_thread.h"
#include "epgsearchtools.h"
#include "changrp.h"
#include "menu_dirselect.h" 
#include "menu_searchtemplate.h"
#include "menu_blacklists.h"
#include "templatefile.h"
#include "conflictcheck_thread.h"

const char *RedKeyMode[2];
const char *BlueKeyMode[2];
const char *ProgressbarMode[4];
const char *StartMenuMode[2];
const char *AddSubtitleMode[3];

// ------------------
// cMenuSetupSubMenu
cMenuSetupSubMenu::cMenuSetupSubMenu(const char* Title, cEPGSearchConfig* Data)
    : cOsdMenu(Title, 33)
{
    data = Data;  
}

eOSState cMenuSetupSubMenu::ProcessKey(eKeys Key)
{
    eOSState state = cOsdMenu::ProcessKey(Key);

    if (state == osUnknown) {
	switch (Key) {
	    case kYellow:
		state = Help();
		break;
	    default: break;
	}
    }

    return state;
}

void cMenuSetupSubMenu::AddHelp(const char* helpText)
{
    helpTexts.push_back(helpText);
}

eOSState cMenuSetupSubMenu::Help()
{
    const char* ItemText = Get(Current())->Text();
    eOSState state = osContinue;
    if(Current() < (int) helpTexts.size())
    {
	char* title = NULL;
	asprintf(&title, "%s - %s", tr("Button$Help"), ItemText);
	if (strchr(title, ':'))
	    *strchr(title, ':') = 0;
	state = AddSubMenu(new cMenuText(title, helpTexts[Current()]));
	free(title);
    }
    return state;
}

// ------------------
// cMenuEPGSearchSetup
cMenuEPGSearchSetup::cMenuEPGSearchSetup(void)
{
    RedKeyMode[0] = tr("Standard");
    RedKeyMode[1] = tr("Commands");
    
    BlueKeyMode[0] = tr("Standard");
    BlueKeyMode[1] = tr("Button$Search");
    
    ProgressbarMode[0] = tr("no");
    ProgressbarMode[1] = tr("pipes");
    ProgressbarMode[2] = tr("graphical");

    StartMenuMode[0] = tr("Button$Schedule");
    StartMenuMode[1] = tr("Button$Now");

    AddSubtitleMode[0] = tr("never");
    AddSubtitleMode[1] = tr("always");
    AddSubtitleMode[2] = tr("smart");

    Setup();
}

void cMenuEPGSearchSetup::Setup(void)
{
    data = EPGSearchConfig;
    if (isempty(EPGSearchConfig.mainmenuentry))
	strcpy(data.mainmenuentry,tr("Program guide"));
    
    Set();
    SetHelp(NULL, NULL, NULL, tr("Button$Open"));
}

void cMenuEPGSearchSetup::Set()
{
  int current = Current();
  Clear();
 
  Add(new cOsdItem(tr("General")));
  Add(new cOsdItem(tr("EPG menus")));
  Add(new cOsdItem(tr("User-defined EPG times")));
  Add(new cOsdItem(tr("Timer programming")));
  Add(new cOsdItem(tr("Search and search timers")));
  Add(new cOsdItem(tr("Timer conflict checking")));

  SetCurrent(Get(current));
  Display();
}

void cMenuEPGSearchSetup::Store(void)
{
    bool RestartSearchTimerThread = false;
    if (EPGSearchConfig.useSearchTimers != data.useSearchTimers)
    {
        RestartSearchTimerThread = true;
	cSearchTimerThread::Exit();
    }

    bool RestartConflictCheckThread = false;
    if (EPGSearchConfig.checkTimerConflictsAfterUpdate != data.checkTimerConflictsAfterUpdate ||
	EPGSearchConfig.conflictCheckIntervall != data.conflictCheckIntervall)
    {
	RestartConflictCheckThread = true;
	cConflictCheckThread::Exit();
    }

    EPGSearchConfig = data;
    if (strcmp(EPGSearchConfig.mainmenuentry, tr("Program guide")) == 0) 
	strcpy(EPGSearchConfig.mainmenuentry,"");
    

    SetupStore("HideMenu",  EPGSearchConfig.hidemenu);
    SetupStore("MainMenuEntry",  EPGSearchConfig.mainmenuentry);
    SetupStore("ReplaceOrgSchedule",  EPGSearchConfig.ReplaceOrgSchedule);
    SetupStore("StartMenu",  EPGSearchConfig.StartMenu);
    SetupStore("RedKeyMode",  EPGSearchConfig.redkeymode);
    SetupStore("BlueKeyMode",  EPGSearchConfig.bluekeymode);
    SetupStore("ShowProgress",  EPGSearchConfig.showProgress);
    SetupStore("ShowChannelNr",  EPGSearchConfig.showChannelNr);
    SetupStore("OnePressTimerCreation",  EPGSearchConfig.onePressTimerCreation);
    
    SetupStore("UserMode1UseIt",  EPGSearchConfig.ShowModes[showUserMode1].GetUsage());
    SetupStore("UserMode1Description",  EPGSearchConfig.ShowModes[showUserMode1].GetDescription());
    SetupStore("UserMode1Time",  EPGSearchConfig.ShowModes[showUserMode1].GetTime());
    
    SetupStore("UserMode2UseIt",  EPGSearchConfig.ShowModes[showUserMode2].GetUsage());
    SetupStore("UserMode2Description",  EPGSearchConfig.ShowModes[showUserMode2].GetDescription());
    SetupStore("UserMode2Time",  EPGSearchConfig.ShowModes[showUserMode2].GetTime());
  
    SetupStore("UserMode3UseIt",  EPGSearchConfig.ShowModes[showUserMode3].GetUsage());
    SetupStore("UserMode3Description",  EPGSearchConfig.ShowModes[showUserMode3].GetDescription());
    SetupStore("UserMode3Time",  EPGSearchConfig.ShowModes[showUserMode3].GetTime());
    
    SetupStore("UserMode4UseIt",  EPGSearchConfig.ShowModes[showUserMode4].GetUsage());
    SetupStore("UserMode4Description",  EPGSearchConfig.ShowModes[showUserMode4].GetDescription());
    SetupStore("UserMode4Time",  EPGSearchConfig.ShowModes[showUserMode4].GetTime());

    SetupStore("UseSearchTimers",  EPGSearchConfig.useSearchTimers);
    SetupStore("UpdateIntervall",  EPGSearchConfig.UpdateIntervall);
    SetupStore("SVDRPPort",  EPGSearchConfig.SVDRPPort);
    SetupStore("CheckTimerConflicts",  EPGSearchConfig.checkTimerConflictsAfterUpdate);
    SetupStore("CheckTimerConflictsPriority",  EPGSearchConfig.checkMinPriority);
    SetupStore("CheckTimerConflictsDays",  EPGSearchConfig.checkMaxDays);
    SetupStore("CheckConflictsIntervall",  EPGSearchConfig.conflictCheckIntervall);
    SetupStore("CheckConflictsWithinLimit",  EPGSearchConfig.conflictCheckWithinLimit);
    SetupStore("CheckConflictsIntervall2",  EPGSearchConfig.conflictCheckIntervall2);
    SetupStore("CheckConflictsMinDuration",  EPGSearchConfig.checkMinDuration);

    SetupStore("TimeIntervallFRFF",  EPGSearchConfig.timeShiftValue);
    SetupStore("ToggleGreenYellow",  EPGSearchConfig.toggleGreenYellow);

    SetupStore("ShowRadioChannels", EPGSearchConfig.showRadioChannels);

    SetupStore("DefPriority",  EPGSearchConfig.DefPriority);
    SetupStore("DefLifetime",  EPGSearchConfig.DefLifetime);
    SetupStore("DefMarginStart",  EPGSearchConfig.DefMarginStart);
    SetupStore("DefMarginStop",  EPGSearchConfig.DefMarginStop);

    SetupStore("IgnorePayTV",  EPGSearchConfig.ignorePayTV);
    SetupStore("DefRecordingDir",  EPGSearchConfig.defrecdir);
    SetupStore("UseVDRTimerEditMenu",  EPGSearchConfig.useVDRTimerEditMenu);
    SetupStore("ShowChannelGroups",  EPGSearchConfig.showChannelGroups);
    SetupStore("ShowDaySeparators",  EPGSearchConfig.showDaySeparators);
    SetupStore("ShowEmptyChannels",  EPGSearchConfig.showEmptyChannels);

    SetupStore("DefSearchTemplateID",  EPGSearchConfig.DefSearchTemplateID);

    SetupStore("AddSubtitleToTimerMode",  EPGSearchConfig.addSubtitleToTimer);

    cTemplFile::Reset();
    char* templateFilename = strdup(AddDirectory(CONFIGDIR, "epgsearchmenu.conf"));
    if (access(templateFilename, F_OK) == 0)
    {
	cTemplFile templFile;
	if (!templFile.Load(templateFilename))
	    esyslog("EPGSearch: could not load %s", templateFilename);
    }
    cTemplFile::PrepareDefaultTemplates();
    free(templateFilename);

    cPluginEpgsearch *p = (cPluginEpgsearch*) cPluginManager::GetPlugin("epgsearch");
    if (RestartSearchTimerThread)
	cSearchTimerThread::Init(p);
    if (RestartConflictCheckThread)
	cConflictCheckThread::Init(p);
}

eOSState cMenuEPGSearchSetup::ProcessKey(eKeys Key)
{
    bool hadSubMenu = HasSubMenu();
    eOSState state = cMenuSetupPage::ProcessKey(Key);

    const char* ItemText = Get(Current())->Text();
    int iOnGeneral = 0;
    int iOnEPGMenus = 0;
    int iOnUserdefTimes = 0;
    int iOnTimers = 0;
    int iOnSearchtimers = 0;
    int iOnTimerConflicts = 0;

    if (!HasSubMenu())
    { 
	if (strstr(ItemText, tr("General")) == ItemText)
	    iOnGeneral = 1;
	else if (strstr(ItemText, tr("EPG menus")) == ItemText)
	    iOnEPGMenus = 1;
	else if (strstr(ItemText, tr("User-defined EPG times")) == ItemText)
	    iOnUserdefTimes = 1;
	else if (strstr(ItemText, tr("Timer programming")) == ItemText)
	    iOnTimers = 1;
	else if (strstr(ItemText, tr("Search and search timers")) == ItemText)
	    iOnSearchtimers = 1;
	else if (strstr(ItemText, tr("Timer conflict checking")) == ItemText)
	    iOnTimerConflicts = 1;
    }

    if (state == osUnknown || Key == kOk) {
	switch (Key) {
	    case kOk:
		if (!hadSubMenu && !HasSubMenu())
		{
		    if (iOnGeneral == 1)
			state = AddSubMenu(new cMenuSetupGeneral(&data));
		    else if (iOnEPGMenus == 1)
			state = AddSubMenu(new cMenuSetupEPGMenus(&data));
		    else if (iOnUserdefTimes == 1)
			state = AddSubMenu(new cMenuSetupUserdefTimes(&data));
		    else if (iOnTimers == 1)
			state = AddSubMenu(new cMenuSetupTimers(&data));
		    else if (iOnSearchtimers == 1)
			state = AddSubMenu(new cMenuSetupSearchtimers(&data));
		    else if (iOnTimerConflicts == 1)
			state = AddSubMenu(new cMenuSetupTimerConflicts(&data));
		}
		break;
	  case kBlue:
	      if (!HasSubMenu())
	      {
		  if (iOnGeneral == 1)
		      state = AddSubMenu(new cMenuSetupGeneral(&data));
		  else if (iOnEPGMenus == 1)
		      state = AddSubMenu(new cMenuSetupEPGMenus(&data));
		  else if (iOnUserdefTimes == 1)
		      state = AddSubMenu(new cMenuSetupUserdefTimes(&data));
		  else if (iOnTimers == 1)
		      state = AddSubMenu(new cMenuSetupTimers(&data));
		  else if (iOnSearchtimers == 1)
		      state = AddSubMenu(new cMenuSetupSearchtimers(&data));
		  else if (iOnTimerConflicts == 1)
		      state = AddSubMenu(new cMenuSetupTimerConflicts(&data));
	      }
	      break;
	  default: break;
      }
    }
    if (!HasSubMenu() && hadSubMenu)
	Store();

    return state;
}

// ------------------
// cMenuSetupGeneral
cMenuSetupGeneral::cMenuSetupGeneral(cEPGSearchConfig* Data)
: cMenuSetupSubMenu(tr("General"), Data)
{
    Set();
}

void cMenuSetupGeneral::Set()
{
  int current = Current();
  Clear();
  helpTexts.clear();

  Add(new cMenuEditBoolItem(tr("Hide main menu entry"),         &data->hidemenu,       tr("no"),      tr("yes")));
  AddHelp(tr("Help$Hides the main menu entry and may be useful if this plugin is used to replace the original 'Schedule' entry."));
  if (!data->hidemenu)
  {
      Add(new cMenuEditStrItem(IndentMenuItem(tr("Main menu entry")), data->mainmenuentry, sizeof(data->mainmenuentry), tr(AllowedChars)));
      AddHelp(tr("Help$The name of the main menu entry which defaults to 'Programm guide'."));
  }
  Add(new cMenuEditBoolItem(tr("Replace original schedule"),    &data->ReplaceOrgSchedule, tr("no"),      tr("yes")));
  AddHelp(tr("Help$When VDR is patched to allow this plugin to replace the original 'Schedule' entry, you can de/activate this replacement here."));
  Add(new cMenuEditStraItem(tr("Start menu"), &data->StartMenu, 2, StartMenuMode));
  AddHelp(tr("Help$Choose between 'Overview - Now' and 'Schedule' as start menu when this plugin is called."));

  SetCurrent(Get(current));
  Display();
  SetHelp(NULL, NULL, tr("Button$Help"), NULL);
}

eOSState cMenuSetupGeneral::ProcessKey(eKeys Key)
{
    int iTemp_hidemenu = data->hidemenu;

    eOSState state = cMenuSetupSubMenu::ProcessKey(Key);

    if (iTemp_hidemenu != data->hidemenu)
    {
	Set();
	Display();
    }

    if (state == osUnknown) {
	switch (Key) {
	    case kOk:
		return osBack;
	    default: break;
	}
    }

    return state;
}



// ------------------
// cMenuSetupEPGMenus
cMenuSetupEPGMenus::cMenuSetupEPGMenus(cEPGSearchConfig* Data)
: cMenuSetupSubMenu(tr("EPG menus"), Data)
{
    Set();
}

void cMenuSetupEPGMenus::Set()
{
    int current = Current();
    Clear();
    helpTexts.clear();
    
    Add(new cMenuEditStraItem(tr("Red key"), &data->redkeymode, 2, RedKeyMode));
    AddHelp(tr("Help$Choose which standard function ('Record' or 'Commands') you like to have on the red key.\n(Can be toggled with key '0')"));
    Add(new cMenuEditStraItem(tr("Blue key"), &data->bluekeymode, 2, BlueKeyMode));
    AddHelp(tr("Help$Choose which standard function ('Switch' or 'Search') you like to have on the blue key.\n(Can be toggled with key '0')"));
    Add(new cMenuEditStraItem(tr("Show progress in 'Now'"),         &data->showProgress, 3, ProgressbarMode));
    AddHelp(tr("Help$Shows a progressbar in 'Overview - Now' that informs about the remaining time of the current event.\nDepending on your selected skin you can choose between 'Pipes' or 'graphical' to get the best look."));
    Add(new cMenuEditBoolItem(  tr("Show channel numbers"),         &data->showChannelNr,       tr("no"),      tr("yes")));
    AddHelp(tr("Help$Display channel numbers in 'Overview - Now'.\n\n(To completely define your own menu look please inspect the MANUAL)"));
    Add(new cMenuEditBoolItem(  tr("Show channel separators"),         &data->showChannelGroups,       tr("no"),      tr("yes")));
    AddHelp(tr("Help$Display VDR channel groups as separators between your channels in 'Overview - Now'."));
    Add(new cMenuEditBoolItem(  tr("Show day separators"),         &data->showDaySeparators,       tr("no"),      tr("yes")));
    AddHelp(tr("Help$Display a separator line at day break in 'Schedule'."));
    Add(new cMenuEditBoolItem(  tr("Show radio channels"),         &data->showRadioChannels,       tr("no"),      tr("yes")));
    AddHelp(tr("Help$Show also radio channels."));
    Add(new cMenuEditBoolItem(  tr("'One press' timer creation"),         &data->onePressTimerCreation,       tr("no"),      tr("yes")));
    AddHelp(tr("Help$When a timer is created with 'Record' you can choose between an immediate creation of the timer or the display of the timer edit menu."));
    Add(new cMenuEditBoolItem(  tr("Show channels without EPG"),         &data->showEmptyChannels,       tr("no"),      tr("yes")));
    AddHelp(tr("Help$Choose 'yes' here if you want to display channels without EPG in 'Overview - Now'. 'Ok' on these entries switches the channel."));
    Add(new cMenuEditIntItem(tr("Time interval for FRew/FFwd [min]"), &data->timeShiftValue, 1, 9999));
    AddHelp(tr("Help$Choose here the time interval which should be used for jumping through the EPG by pressing FRew/FFwd.\n\n(If you don't have those keys, you can toggle to this functionality pressing '0' and get '<<' and '>>' on the keys green and yellow)"));
    Add(new cMenuEditBoolItem(  tr("Toggle Green/Yellow"),         &data->toggleGreenYellow,       tr("no"),      tr("yes")));
    AddHelp(tr("Help$Specify if green and yellow shall also be switched when pressing '0'."));
    
    SetCurrent(Get(current));
    Display();
    SetHelpKeys();
}

void cMenuSetupEPGMenus::SetHelpKeys()
{
    const char* ItemText = Get(Current())->Text();
    if (!HasSubMenu())
    {
	if (strstr(ItemText, tr("Default recording dir")) == ItemText)
	{
	    if (!InEditMode(ItemText, tr("Default recording dir"), data->defrecdir))
		SetHelp(NULL, NULL, tr("Button$Help"), tr("Button$Select"));
	}
	else
	    SetHelp(NULL, NULL, tr("Button$Help"), NULL);
    }
}

eOSState cMenuSetupEPGMenus::ProcessKey(eKeys Key)
{
    int iTemp_showProgress = data->showProgress;

    eOSState state = cMenuSetupSubMenu::ProcessKey(Key);

    const char* ItemText = Get(Current())->Text();
    int iOnDefRecDir = 0;
    if (!HasSubMenu())
	if (strstr(ItemText, tr("Default recording dir")) == ItemText)
	    iOnDefRecDir = 1;

    SetHelpKeys();

    if (iTemp_showProgress != data->showProgress)
    {
	Set();
	Display();
    }

    if (state == osUnknown) {
	switch (Key) {
	    case kBlue:
		if (!HasSubMenu())
		{
		    if (iOnDefRecDir == 1)
			state = AddSubMenu(new cMenuDirSelect(data->defrecdir));
		}
		break;
	    case kOk:
		return osBack;
	    default: break;
	}
    }
    return state;
}

// ------------------
// cMenuSetupUsedefTimes
cMenuSetupUserdefTimes::cMenuSetupUserdefTimes(cEPGSearchConfig* Data)
: cMenuSetupSubMenu(tr("User-defined EPG times"), Data)
{
    Set();
}

void cMenuSetupUserdefTimes::Set()
{
  int current = Current();
  Clear();
  helpTexts.clear();

  char* szUseUserTime = NULL;
  asprintf(&szUseUserTime, "%s %d", tr("Use user-defined time"), 1);
  Add(new cMenuEditBoolItem(szUseUserTime, &data->ShowModes[showUserMode1].useIt,       tr("no"),      tr("yes")));
  AddHelp(tr("Help$Besides 'Now' and 'Next' you can specify up to 4 other times in the EPG which can be used by repeatedly pressing the green key, e.g. 'prime time', 'late night',..."));
  if (data->ShowModes[showUserMode1].GetUsage())
  {
      Add(new cMenuEditStrItem(IndentMenuItem(tr("Description")), data->ShowModes[showUserMode1].description, sizeof(data->ShowModes[showUserMode1].description), tr(FileNameChars)));
      AddHelp(tr("Help$This is the description for your user-defined time as it will appear as label on the green button."));
      Add(new cMenuEditTimeItem(IndentMenuItem(tr("Time")), &data->ShowModes[showUserMode1].itime));
      AddHelp(tr("Help$Specify the user-defined time here in 'HH:MM'."));
  } 
  free(szUseUserTime);

  asprintf(&szUseUserTime, "%s %d", tr("Use user-defined time"), 2);
  Add(new cMenuEditBoolItem( szUseUserTime, &data->ShowModes[showUserMode2].useIt,       tr("no"),      tr("yes")));
  AddHelp(tr("Help$Besides 'Now' and 'Next' you can specify up to 4 other times in the EPG which can be used by repeatedly pressing the green key, e.g. 'prime time', 'late night',..."));
  if (data->ShowModes[showUserMode2].GetUsage())
  {
      Add(new cMenuEditStrItem(IndentMenuItem(tr("Description")), data->ShowModes[showUserMode2].description, sizeof(data->ShowModes[showUserMode2].description), tr(FileNameChars)));
      AddHelp(tr("Help$This is the description for your user-defined time as it will appear as label on the green button."));
      Add(new cMenuEditTimeItem(IndentMenuItem(tr("Time")), &data->ShowModes[showUserMode2].itime));
      AddHelp(tr("Help$Specify the user-defined time here in 'HH:MM'."));
  } 
  free(szUseUserTime);

  asprintf(&szUseUserTime, "%s %d", tr("Use user-defined time"), 3);
  Add(new cMenuEditBoolItem(szUseUserTime, &data->ShowModes[showUserMode3].useIt,       tr("no"),      tr("yes")));
  AddHelp(tr("Help$Besides 'Now' and 'Next' you can specify up to 4 other times in the EPG which can be used by repeatedly pressing the green key, e.g. 'prime time', 'late night',..."));
  if (data->ShowModes[showUserMode3].GetUsage())
  {
      Add(new cMenuEditStrItem(IndentMenuItem(tr("Description")), data->ShowModes[showUserMode3].description, sizeof(data->ShowModes[showUserMode3].description), tr(FileNameChars)));
      AddHelp(tr("Help$This is the description for your user-defined time as it will appear as label on the green button."));
      Add(new cMenuEditTimeItem(IndentMenuItem(tr("Time")), &data->ShowModes[showUserMode3].itime));
      AddHelp(tr("Help$Specify the user-defined time here in 'HH:MM'."));
  } 
  free(szUseUserTime);

  asprintf(&szUseUserTime, "%s %d", tr("Use user-defined time"), 4);
  Add(new cMenuEditBoolItem(szUseUserTime, &data->ShowModes[showUserMode4].useIt,       tr("no"),      tr("yes")));
  AddHelp(tr("Help$Besides 'Now' and 'Next' you can specify up to 4 other times in the EPG which can be used by repeatedly pressing the green key, e.g. 'prime time', 'late night',..."));
  if (data->ShowModes[showUserMode4].GetUsage())
  {
      Add(new cMenuEditStrItem(IndentMenuItem(tr("Description")), data->ShowModes[showUserMode4].description, sizeof(data->ShowModes[showUserMode4].description), tr(FileNameChars)));
      AddHelp(tr("Help$This is the description for your user-defined time as it will appear as label on the green button."));
      Add(new cMenuEditTimeItem(IndentMenuItem(tr("Time")), &data->ShowModes[showUserMode4].itime));
      AddHelp(tr("Help$Specify the user-defined time here in 'HH:MM'."));
  } 
  free(szUseUserTime);

  SetCurrent(Get(current));
  Display();
  SetHelp(NULL, NULL, tr("Button$Help"), NULL);
}

eOSState cMenuSetupUserdefTimes::ProcessKey(eKeys Key)
{
    int iTemp_useUserTime1 = data->ShowModes[showUserMode1].GetUsage();
    int iTemp_useUserTime2 = data->ShowModes[showUserMode2].GetUsage();
    int iTemp_useUserTime3 = data->ShowModes[showUserMode3].GetUsage();
    int iTemp_useUserTime4 = data->ShowModes[showUserMode4].GetUsage();

    eOSState state = cMenuSetupSubMenu::ProcessKey(Key);

    if (iTemp_useUserTime1 != data->ShowModes[showUserMode1].GetUsage() ||
	iTemp_useUserTime2 != data->ShowModes[showUserMode2].GetUsage() ||
	iTemp_useUserTime3 != data->ShowModes[showUserMode3].GetUsage() ||
	iTemp_useUserTime4 != data->ShowModes[showUserMode4].GetUsage())
    {
	Set();
	Display();
    }

    if (state == osUnknown) {
	switch (Key) {
	    case kOk:
		return osBack;
	    default: break;
	}
    }

    return state;
}


// ------------------
// cMenuSetupTimers
cMenuSetupTimers::cMenuSetupTimers(cEPGSearchConfig* Data)
: cMenuSetupSubMenu(tr("Timer programming"), Data)
{
    Set();
}

void cMenuSetupTimers::Set()
{
  int current = Current();
  Clear();
  helpTexts.clear();

  Add(new cMenuEditBoolItem(  tr("Use VDR's timer edit menu"),         &data->useVDRTimerEditMenu,       tr("no"),      tr("yes")));
  AddHelp(tr("Help$This plugin has it's own timer edit menu extending the original one with some extra functionality like\n- an addtional directory entry\n- user-defined days of week for repeating timers\n- adding an episode name\n- support for EPG variables (see MANUAL)"));
  Add(new cMenuEditStrItem(tr("Default recording dir"), data->defrecdir, sizeof(data->defrecdir), tr(AllowedChars)));
  AddHelp(tr("Help$When creating a timer you can specify here a default recording directory."));
  Add(new cMenuEditStraItem(tr("Add episode to manual timers"), &data->addSubtitleToTimer, 3, AddSubtitleMode));
  AddHelp(tr("Help$If you create a timer for a series, you can automatically add the episode name.\n\n- never: no addition\n- always: always add episode name if present\n- smart: add only if event lasts less than 80 mins."));

  SetCurrent(Get(current));
  Display();
  SetHelpKeys();
}

void cMenuSetupTimers::SetHelpKeys()
{
    const char* ItemText = Get(Current())->Text();
    if (!HasSubMenu())
    {
	if (strstr(ItemText, tr("Default recording dir")) == ItemText)
	{
	    if (!InEditMode(ItemText, tr("Default recording dir"), data->defrecdir))
		SetHelp(NULL, NULL, tr("Button$Help"), tr("Button$Select"));
	}
	else
	    SetHelp(NULL, NULL, tr("Button$Help"), NULL);
    }
}

eOSState cMenuSetupTimers::ProcessKey(eKeys Key)
{
    eOSState state = cMenuSetupSubMenu::ProcessKey(Key);

    const char* ItemText = Get(Current())->Text();
    int iOnDefRecDir = 0;
    if (!HasSubMenu())
    { 
	if (strstr(ItemText, tr("Default recording dir")) == ItemText)
	    iOnDefRecDir = 1;
    }
    SetHelpKeys();

    if (state == osUnknown) {
	switch (Key) {
	  case kBlue:
	      if (!HasSubMenu())
	      {
		  if (iOnDefRecDir == 1)
		      state = AddSubMenu(new cMenuDirSelect(data->defrecdir));
	      }
	      break;
	    case kOk:
		return osBack;
	    default: break;
	}
    }

    return state;
}

// ------------------
// cMenuSetupSearchtimers
cMenuSetupSearchtimers::cMenuSetupSearchtimers(cEPGSearchConfig* Data)
: cMenuSetupSubMenu(tr("Search and search timers"), Data)
{
    Set();
}

void cMenuSetupSearchtimers::Set()
{
  int current = Current();
  Clear();
  helpTexts.clear();

  Add(new cMenuEditBoolItem(tr("Use search timers"), &data->useSearchTimers, tr("no"), tr("yes")));
  AddHelp(tr("Help$'Search timers' can be used to automatically create timers for events that match your search criterions."));
  if (data->useSearchTimers)
  {
      Add(new cMenuEditIntItem(tr("  Update interval [min]"), &data->UpdateIntervall, 1, 9999));
      AddHelp(tr("Help$Specify here the time intervall to be used when searching for events in the background."));
      Add(new cMenuEditIntItem(tr("  SVDRP port"), &data->SVDRPPort, 1, 99999));
      AddHelp(tr("Help$Programming of new timers or timer changes is done with SVDRP. The default value should be correct, so change it only if you know what you are doing."));
      Add(new cMenuEditIntItem(IndentMenuItem(tr("Setup.Recording$Default priority")), &data->DefPriority, 0, MAXPRIORITY));
      AddHelp(tr("Help$Specify here the default priority of timers created with this plugin. This value can also be adjusted for each search itself."));
      Add(new cMenuEditIntItem(IndentMenuItem(tr("Setup.Recording$Default lifetime (d)")), &data->DefLifetime, 0, MAXLIFETIME));
      AddHelp(tr("Help$Specify here the default lifetime of timers/recordings created with this plugin. This value can also be adjusted for each search itself."));
      Add(new cMenuEditIntItem(IndentMenuItem(tr("Setup.Recording$Margin at start (min)")), &data->DefMarginStart));
      AddHelp(tr("Help$Specify here the default start recording margin of timers/recordings created with this plugin. This value can also be adjusted for each search itself."));
      Add(new cMenuEditIntItem(IndentMenuItem(tr("Setup.Recording$Margin at stop (min)")), &data->DefMarginStop));
      AddHelp(tr("Help$Specify here the default stop recording margin of timers/recordings created with this plugin. This value can also be adjusted for each search itself."));
  }

  Add(new cMenuEditBoolItem(  tr("Ignore PayTV channels"), &data->ignorePayTV, tr("no"), tr("yes")));  
  AddHelp(tr("Help$Set this to 'yes' if don't want to see events on PayTV channels when searching for repeats."));
  Add(new cOsdItem(tr("Search templates")));
  AddHelp(tr("Help$Here you can setup templates for your searches."));
  Add(new cOsdItem(tr("Blacklists")));
  AddHelp(tr("Help$Here you can setup blacklists which can be used within a search to exclude events you don't like."));
  Add(new cOsdItem(tr("Channel groups")));
  AddHelp(tr("Help$Here you can setup channel groups which can be used within a search. These are different to VDR channel groups and represent a set of arbitrary channels, e.g. 'FreeTV'."));

  SetCurrent(Get(current));
  Display();
  SetHelpKeys();
}

void cMenuSetupSearchtimers::SetHelpKeys()
{
    const char* ItemText = Get(Current())->Text();
    if (!HasSubMenu())
    {
	if (strstr(ItemText, tr("Channel groups")) == ItemText)
	    SetHelp(NULL, NULL, tr("Button$Help"), tr("Button$Setup"));
	else if (strstr(ItemText, tr("Search templates")) == ItemText)
	    SetHelp(NULL, NULL, tr("Button$Help"), tr("Button$Setup"));
	else if (strstr(ItemText, tr("Blacklists")) == ItemText)
	    SetHelp(NULL, NULL, tr("Button$Help"), tr("Button$Setup"));
	else
	    SetHelp(NULL, NULL, tr("Button$Help"), NULL);
    }
}

eOSState cMenuSetupSearchtimers::ProcessKey(eKeys Key)
{
    int iTemp_useSearchTimers = data->useSearchTimers;

    int iOnSearchTemplates = 0;
    int iOnBlacklists = 0;
    int iOnChannelGroups = 0;

    eOSState state = cMenuSetupSubMenu::ProcessKey(Key);

    if (iTemp_useSearchTimers != data->useSearchTimers)
    {
	Set();
	Display();
    }

    const char* ItemText = Get(Current())->Text();
    if (!HasSubMenu())
    { 
	if (strstr(ItemText, tr("Search templates")) == ItemText)
	    iOnSearchTemplates = 1;
	else if (strstr(ItemText, tr("Blacklists")) == ItemText)
	    iOnBlacklists = 1;
	if (strstr(ItemText, tr("Channel groups")) == ItemText)
	    iOnChannelGroups = 1;
    }
    SetHelpKeys();

    if (state == osUnknown) {
	switch (Key) {
	    case kBlue:
		if (!HasSubMenu())
		{
		    if (iOnSearchTemplates == 1)
			state = AddSubMenu(new cMenuEPGSearchTemplate(NULL, NULL, false));
		    else if (iOnBlacklists == 1)
			state = AddSubMenu(new cMenuBlacklists);
		    else if (iOnChannelGroups == 1)
			state = AddSubMenu(new cMenuChannelGroups);
		}
		break;
	    case kOk:
		return osBack;
	    default: break;
	}
    }
    return state;
}

// ------------------
// cMenuSetupTimerConflicts
cMenuSetupTimerConflicts::cMenuSetupTimerConflicts(cEPGSearchConfig* Data)
: cMenuSetupSubMenu(tr("Timer conflict checking"), Data)
{
    Set();
}

void cMenuSetupTimerConflicts::Set()
{
  int current = Current();
  Clear();
  helpTexts.clear();

  Add(new cMenuEditIntItem(tr("Ignore below priority"), &data->checkMinPriority, 0, MAXPRIORITY));
  AddHelp(tr("Help$If a timer with priority below the given value will fail it will not be classified as important. Only important conflicts will produce an OSD message about the conflict after an automatic conflict check."));
  Add(new cMenuEditIntItem(tr("Ignore conflict duration less ... min."), &data->checkMinDuration, 0, 999));
  AddHelp(tr("Help$If a conflicts duration is less then the given number of minutes it will not be classified as important. Only important conflicts will produce an OSD message about the conflict after an automatic conflict check."));
  Add(new cMenuEditIntItem(tr("Only check within next ... days"), &data->checkMaxDays, 1, 99));
  AddHelp(tr("Help$This value reduces the conflict check to the given range of days. All other conflicts are classified as 'not yet important'."));

  cOsdItem* sep = new cOsdItem(tr("--- Automatic checking ---"));
  sep->SetSelectable(false);
  Add(sep);
  AddHelp("dummy");
  Add(new cMenuEditBoolItem(tr("After each search timer update"),         &data->checkTimerConflictsAfterUpdate,       tr("no"),      tr("yes")));
  AddHelp(tr("Help$Set this to 'yes' if the conflict check should be performed after each search timer update."));
  if (!data->checkTimerConflictsAfterUpdate)
  {
      Add(new cMenuEditIntItem(IndentMenuItem(tr("every ... minutes")), &data->conflictCheckIntervall, 0, 999));
      AddHelp(tr("Help$Specify here the time intervall to be used for an automatic conflict check in the background.\n('0' disables an automatic check)"));
      Add(new cMenuEditIntItem(IndentMenuItem(tr("if conflicts within next ... minutes")), &data->conflictCheckWithinLimit, 0, 9999));
      AddHelp(tr("Help$If the next conflict will appear in the given number of minutes you can specify here a shorter check intervall to get more OSD notifications about it."));
      if (data->conflictCheckWithinLimit)
      {
	  Add(new cMenuEditIntItem(IndentMenuItem(IndentMenuItem(tr("every ... minutes"))), &data->conflictCheckIntervall2, 1, 999));
      AddHelp(tr("Help$If the next conflict will appear in the given number of minutes you can specify here a shorter check intervall to get more OSD notifications about it."));
      }
  }

  SetCurrent(Get(current));
  Display();
  SetHelp(NULL, NULL, tr("Button$Help"), NULL);
}

eOSState cMenuSetupTimerConflicts::ProcessKey(eKeys Key)
{
    int iTemp_checkTimerConflictsAfterUpdate = data->checkTimerConflictsAfterUpdate;
    int iTemp_conflictCheckWithinLimit = data->conflictCheckWithinLimit;

    eOSState state = cMenuSetupSubMenu::ProcessKey(Key);

    if (iTemp_checkTimerConflictsAfterUpdate != data->checkTimerConflictsAfterUpdate ||
	(iTemp_conflictCheckWithinLimit != data->conflictCheckWithinLimit && (iTemp_conflictCheckWithinLimit == 0 || data->conflictCheckWithinLimit == 0)))
    {
	Set();
	Display();
	state = ProcessKey(Key);
    }

    if (state == osUnknown) {
	switch (Key) {
	    case kOk:
		return osBack;
	    default: break;
	}
    }

    return state;
}

